package main

import (
	"context"
	"fmt"
	"github.com/rs/cors"
	"go.opentelemetry.io/otel"
	"log"
	"model/config"
	"model/diagnostics"
	"model/internal/kafka"
	"model/internal/repository"
	"net/http"
	"os/signal"
	"syscall"

	"golang.org/x/sync/errgroup"

	"model/internal/handler"
	"model/internal/productassistant"
	"model/internal/service"
)

func main() {
	// Загружаем конфигурацию из переменных окружения.
	cfg, err := config.NewConfig(context.Background())
	if err != nil {
		log.Fatalf("failed to load config: %v", err)
	}

	// Создаем контекст для graceful shutdown.
	ctx, stop := signal.NotifyContext(context.Background(), syscall.SIGINT, syscall.SIGTERM)
	defer stop()

	// Инициализируем OpenTelemetry-трейсер.
	otelProvider, err := diagnostics.NewOTel(ctx, cfg.OTEL, cfg.Server.AppName)
	if err != nil {
		log.Fatalf("failed to initialize tracer: %v", err)
	}
	// Устанавливаем глобальный провайдер и гарантируем его закрытие при завершении.
	otel.SetTracerProvider(otelProvider.TracerProvider)
	defer func() {
		if err := otelProvider.Close(ctx); err != nil {
			log.Printf("failed to shutdown tracer provider: %v", err)
		}
	}()

	// Инициализируем Kafka для уведомлений.
	kafkaInstance, err := kafka.InitKafka(&cfg.Kafka)
	if err != nil {
		log.Fatalf("failed to initialize Kafka: %v", err)
	}
	producer, err := kafkaInstance.Producer(cfg.Kafka.Producer)
	if err != nil {
		log.Fatalf("failed to create Kafka producer: %v", err)
	}
	consumer, err := kafkaInstance.Consumer(cfg.Kafka.Consumer)
	if err != nil {
		log.Fatalf("failed to create Kafka consumer: %v", err)
	}
	// Создаем репозиторий для доступа к данным.
	storeRepo, err := repository.NewStoreRepository(cfg)
	if err != nil {
		log.Fatalf("failed to create store repository: %v", err)
	}
	// Создаем TelegramNotifier, используя токен из конфигурации.
	telegramNotifier := service.NewTelegramNotifier(cfg.Telegram.BotToken)

	// Инициализируем сервисы.
	productService, err := service.NewProductService(ctx, cfg)
	if err != nil {
		log.Fatalf("failed to init product service: %v", err)
	}
	llmService, err := service.NewLLMService(ctx, cfg, productService)
	if err != nil {
		log.Fatalf("failed to init LLM service: %v", err)
	}
	favService, err := service.NewFavouriteService(ctx, cfg)
	if err != nil {
		log.Fatalf("failed to init favourite service: %v", err)
	}
	telegramAuthService := service.NewTelegramAuthService(cfg)
	notificationService := service.NewNotificationService(producer, consumer, storeRepo, telegramNotifier, cfg.Kafka.Producer.Topic)

	// Создаем сервер приложения.
	assistantServer, err := productassistant.NewServer(cfg, telegramAuthService, productService, llmService, favService, notificationService)
	if err != nil {
		log.Fatalf("failed to create assistant server: %v", err)
	}

	// Передаем jwtSecret (берем из конфигурации).
	httpHandler := handler.AppHandler(assistantServer, cfg.Server.Secret)

	// Настраиваем CORS:
	corsHandler := cors.New(cors.Options{
		AllowedOrigins:   []string{"*"},
		AllowedMethods:   []string{"GET", "POST", "PUT", "DELETE", "OPTIONS"},
		AllowedHeaders:   []string{"Authorization", "Content-Type"},
		AllowCredentials: true,
	}).Handler(httpHandler)

	addr := fmt.Sprintf(":%s", cfg.Server.Port)

	// Запускаем HTTP-сервер и консьюмера параллельно.
	g, ctx := errgroup.WithContext(ctx)
	g.Go(func() error {
		log.Printf("Starting server on %s", addr)
		return http.ListenAndServe(addr, corsHandler)
	})
	g.Go(func() error {
		notificationService.StartConsumer(ctx)
		<-ctx.Done()
		return nil
	})
	g.Go(func() error {
		<-ctx.Done()

		if shTraceProvider, ok := otelProvider.TracerProvider.(interface {
			Shutdown(ctx context.Context) error
		}); ok {
			return shTraceProvider.Shutdown(ctx)
		}

		return nil
	})

	if err := g.Wait(); err != nil {
		log.Printf("Server terminated: %v", err)
	}
	log.Println("Shutdown complete")
}
