package diagnostics

import (
	"context"
	"encoding/hex"
	"fmt"
	"strings"

	"github.com/MrAlias/otel-schema-utils/schema"
	"go.opentelemetry.io/otel"
	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/exporters/otlp/otlptrace"
	"go.opentelemetry.io/otel/exporters/otlp/otlptrace/otlptracegrpc"
	"go.opentelemetry.io/otel/exporters/otlp/otlptrace/otlptracehttp"
	"go.opentelemetry.io/otel/sdk/resource"
	sdktrace "go.opentelemetry.io/otel/sdk/trace"
	semconv "go.opentelemetry.io/otel/semconv/v1.26.0"
	"go.opentelemetry.io/otel/trace"
	"go.opentelemetry.io/otel/trace/noop"

	"model/config"
)

const (
	ServiceNameKey = attribute.Key("serviceName")
	ServiceEnvKey  = attribute.Key("service.env")
)

type OpenTelemetry struct {
	trace.TracerProvider
}

func NewOTel(ctx context.Context, cfg config.OTelConfig, appName string) (OpenTelemetry, error) {
	// Use no-op provider if no otlp url specified to omit warning messages from real tracer provider.
	if cfg.OTLP.Endpoint == "" {
		return OpenTelemetry{
			TracerProvider: noop.NewTracerProvider(),
		}, nil
	}

	var (
		exporter *otlptrace.Exporter
		err      error
	)

	switch cfg.OTLP.HTTP {
	case true:
		exporter, err = NewHTTPExporter(ctx, cfg)
	case false:
		exporter, err = NewGrpcExporter(ctx, cfg)
	}

	if err != nil {
		return OpenTelemetry{}, err
	}

	res, err := constructResource(ctx, appName)
	if err != nil {
		return OpenTelemetry{}, fmt.Errorf("failed to create resource: %w", err)
	}

	tracerProvider := sdktrace.NewTracerProvider(
		sdktrace.WithBatcher(exporter),
		sdktrace.WithResource(res),
	)
	otel.SetTracerProvider(tracerProvider)

	return OpenTelemetry{
		TracerProvider: tracerProvider,
	}, nil
}

func (o *OpenTelemetry) Start(ctx context.Context) {
	<-ctx.Done()

	if shTraceProvider, ok := o.TracerProvider.(interface {
		Shutdown(ctx context.Context) error
	}); ok {
		_ = shTraceProvider.Shutdown(ctx)
	}
}

func (o *OpenTelemetry) Close(ctx context.Context) error {
	if shutdownTraceProvider, ok := o.TracerProvider.(interface {
		Shutdown(ctx context.Context) error
	}); ok {
		return shutdownTraceProvider.Shutdown(ctx)
	}

	return nil
}

func constructResource(ctx context.Context, appName string) (*resource.Resource, error) {
	const targetSchemaURL = semconv.SchemaURL

	conv := schema.NewConverter(nil)

	r, err := conv.Resource(ctx, targetSchemaURL, resource.Default())
	if err != nil {
		return nil, fmt.Errorf("failed to convert default resource: %w", err)
	}

	res, err := resource.Merge(
		r,
		resource.NewWithAttributes(
			targetSchemaURL,
			semconv.ServiceName(appName),
			ServiceNameKey.String(appName),
			ServiceEnvKey.String(appName),
		),
	)
	if err != nil {
		return nil, fmt.Errorf("failed to merge resources: %w", err)
	}

	return res, nil
}

func NewGrpcExporter(ctx context.Context, cfg config.OTelConfig) (*otlptrace.Exporter, error) {
	rc := otlptracegrpc.RetryConfig{
		Enabled:         cfg.OTLP.Retry.Enabled,
		InitialInterval: cfg.OTLP.Retry.InitialInterval,
		MaxInterval:     cfg.OTLP.Retry.MaxInterval,
		MaxElapsedTime:  cfg.OTLP.Retry.MaxElapsedTime,
	}

	opts := []otlptracegrpc.Option{
		otlptracegrpc.WithEndpoint(cfg.OTLP.Endpoint),
		otlptracegrpc.WithTimeout(cfg.OTLP.Timeout),
		otlptracegrpc.WithRetry(rc),
	}

	if cfg.OTLP.Insecure {
		opts = append(opts, otlptracegrpc.WithInsecure())
	}

	exporter, err := otlptracegrpc.New(ctx, opts...)
	if err != nil {
		return nil, fmt.Errorf("new otlpgrpc exporter: %w", err)
	}

	return exporter, nil
}

func NewHTTPExporter(ctx context.Context, cfg config.OTelConfig) (*otlptrace.Exporter, error) {
	rc := otlptracehttp.RetryConfig{
		Enabled:         cfg.OTLP.Retry.Enabled,
		InitialInterval: cfg.OTLP.Retry.InitialInterval,
		MaxInterval:     cfg.OTLP.Retry.MaxInterval,
		MaxElapsedTime:  cfg.OTLP.Retry.MaxElapsedTime,
	}

	opts := []otlptracehttp.Option{
		otlptracehttp.WithEndpoint(cfg.OTLP.Endpoint),
		otlptracehttp.WithTimeout(cfg.OTLP.Timeout),
		otlptracehttp.WithRetry(rc),
	}

	if cfg.OTLP.Insecure {
		opts = append(opts, otlptracehttp.WithInsecure())
	}

	exporter, err := otlptracehttp.New(ctx, opts...)
	if err != nil {
		return nil, fmt.Errorf("new otlphttp exporter: %w", err)
	}

	return exporter, nil
}

func TraceParentFromSpanContext(sc trace.SpanContext) string {
	if !sc.IsValid() {
		return ""
	}

	const version = 0

	var (
		sb  strings.Builder
		buf [32]byte
	)

	// Clear all flags other than the trace-context supported sampling bit.
	flags := sc.TraceFlags() & trace.FlagsSampled

	sb.Grow(2 + 32 + 16 + 2 + 3)
	_, _ = sb.WriteString(fmt.Sprintf("%.2X", version))

	traceID := sc.TraceID()
	spanID := sc.SpanID()
	flagByte := [1]byte{byte(flags)}

	for _, src := range [][]byte{traceID[:], spanID[:], flagByte[:]} {
		_ = sb.WriteByte('-')
		n := hex.Encode(buf[:], src)
		_, _ = sb.Write(buf[:n])
	}

	return sb.String()
}
