package auth

import (
	"context"
	"errors"
	"fmt"
	"net/http"
	"strings"

	"github.com/golang-jwt/jwt/v4"
)

type contextKey string

const userContextKey contextKey = "user_id"

// JWTAuthMiddleware проверяет JWT-токен в заголовке Authorization и кладёт user_id в контекст.
func JWTAuthMiddleware(jwtSecret string) func(http.Handler) http.Handler {
	return func(next http.Handler) http.Handler {
		return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
			authHeader := r.Header.Get("Authorization")
			if authHeader == "" {
				http.Error(w, "missing Authorization header", http.StatusUnauthorized)
				return
			}
			parts := strings.SplitN(authHeader, " ", 2)
			if len(parts) != 2 || strings.ToLower(parts[0]) != "bearer" {
				http.Error(w, "invalid Authorization header", http.StatusUnauthorized)
				return
			}
			tokenString := parts[1]
			token, err := jwt.Parse(tokenString, func(token *jwt.Token) (interface{}, error) {
				if _, ok := token.Method.(*jwt.SigningMethodHMAC); !ok {
					return nil, errors.New("unexpected signing method")
				}
				return []byte(jwtSecret), nil
			})
			if err != nil || !token.Valid {
				http.Error(w, "invalid token", http.StatusUnauthorized)
				return
			}
			claims, ok := token.Claims.(jwt.MapClaims)
			if !ok {
				http.Error(w, "invalid token claims", http.StatusUnauthorized)
				return
			}
			uid, ok := claims["user_id"]
			if !ok {
				http.Error(w, "user_id not present in token", http.StatusUnauthorized)
				return
			}
			var userID string
			switch v := uid.(type) {
			case float64:
				userID = fmt.Sprintf("%.0f", v)
			case string:
				userID = v
			default:
				http.Error(w, "invalid user_id type", http.StatusUnauthorized)
				return
			}
			ctx := context.WithValue(r.Context(), userContextKey, userID)
			next.ServeHTTP(w, r.WithContext(ctx))
		})
	}
}

// GetUserIDFromContext извлекает user_id из контекста.
func GetUserIDFromContext(ctx context.Context) (string, bool) {
	uid, ok := ctx.Value(userContextKey).(string)
	return uid, ok
}
