package kafka

import (
	"context"
	"crypto/tls"
	"crypto/x509"
	"fmt"
	"model/config"
	"os"
	"sync"
	"time"

	"github.com/segmentio/kafka-go"
)

// Message – тип сообщения Kafka (alias для kafka.Message).
type Message = kafka.Message

// Producer – интерфейс продюсера.
type Producer interface {
	// Write отправляет сообщения синхронно.
	Write(ctx context.Context, messages ...Message) error
	// SendMessage отправляет сообщение асинхронно.
	SendMessage(ctx context.Context, msg *Message)
	// Close закрывает продюсера.
	Close() error
}

// Consumer – интерфейс консьюмера.
type Consumer interface {
	// Start запускает обработку сообщений с использованием обработчика.
	Start(ctx context.Context, handler MessageHandler)
	// Fetch читает одно сообщение.
	Fetch(ctx context.Context) (Message, error)
	// Commit подтверждает обработку сообщений.
	Commit(ctx context.Context, messages ...Message) error
	// Close закрывает консьюмера.
	Close() error
}

// MessageHandler – функция-обработчик сообщений.
type MessageHandler func(ctx context.Context, msg Message) error

// Kafka – агрегатор продюсеров и консьюмеров.
type Kafka struct {
	bootstrapServers []string
	producers        []Producer
	consumers        []Consumer
	tlsCfg           *tls.Config
	mu               sync.Mutex
}

// InitKafka инициализирует Kafka с использованием конфигурации проекта.
func InitKafka(cfg *config.KafkaConfig) (*Kafka, error) {
	tlsConf, err := tlsConfig(cfg)
	if err != nil {
		return nil, fmt.Errorf("failed to build kafka tls config: %w", err)
	}
	return &Kafka{
		bootstrapServers: cfg.BootstrapServers,
		producers:        make([]Producer, 0),
		consumers:        make([]Consumer, 0),
		tlsCfg:           tlsConf,
	}, nil
}

func tlsConfig(cfg *config.KafkaConfig) (*tls.Config, error) {
	if cfg.SSLCertPath == "" {
		return nil, nil
	}
	keypair, err := tls.LoadX509KeyPair(cfg.SSLCertPath, cfg.SSLKeyPath)
	if err != nil {
		return nil, err
	}
	caCert, err := os.ReadFile(cfg.SSLCaCert)
	if err != nil {
		return nil, err
	}
	caCertPool := x509.NewCertPool()
	if ok := caCertPool.AppendCertsFromPEM(caCert); !ok {
		return nil, fmt.Errorf("failed to parse CA Cert file")
	}
	return &tls.Config{
		Certificates:       []tls.Certificate{keypair},
		RootCAs:            caCertPool,
		MinVersion:         tls.VersionTLS12,
		InsecureSkipVerify: !cfg.VerifyCert,
	}, nil
}

// Producer создает нового продюсера Kafka с указанной конфигурацией.
func (k *Kafka) Producer(cfg config.ProducerConfig) (Producer, error) {
	writer := &kafka.Writer{
		Addr:                   kafka.TCP(k.bootstrapServers...),
		Topic:                  cfg.Topic,
		Async:                  cfg.Async,
		AllowAutoTopicCreation: cfg.AllowAutoTopicCreation,
		Transport: &kafka.Transport{
			DialTimeout: 10 * time.Second,
			TLS:         k.tlsCfg,
		},
		MaxAttempts: 10,
	}
	producer := &producerWrapper{
		writer: writer,
	}
	k.mu.Lock()
	k.producers = append(k.producers, producer)
	k.mu.Unlock()
	return producer, nil
}

// Consumer создает нового консьюмера Kafka с указанной конфигурацией.
func (k *Kafka) Consumer(cfg config.ConsumerConfig) (Consumer, error) {
	readerConfig := kafka.ReaderConfig{
		Brokers: k.bootstrapServers,
		GroupID: cfg.GroupID,
		Topic:   cfg.Topic,
		Dialer: &kafka.Dialer{
			Timeout:   10 * time.Second,
			DualStack: true,
			TLS:       k.tlsCfg,
		},
		MaxAttempts: 3,
	}
	consumer := &consumerWrapper{
		reader: kafka.NewReader(readerConfig),
	}
	k.mu.Lock()
	k.consumers = append(k.consumers, consumer)
	k.mu.Unlock()
	return consumer, nil
}

// Shutdown закрывает все продюсеры и консьюмеры.
func (k *Kafka) Shutdown(ctx context.Context) {
	k.mu.Lock()
	defer k.mu.Unlock()
	for _, consumer := range k.consumers {
		_ = consumer.Close()
	}
	for _, producer := range k.producers {
		_ = producer.Close()
	}
}
