package productassistant

import (
	"context"
	"encoding/json"
	"fmt"
	"go.opentelemetry.io/otel"
	"model/config"
	"model/internal/auth"
	"model/internal/domain"
	"model/internal/helper"
	"model/internal/repository"
	"model/internal/service"
	"time"
)

//go:generate oapi-codegen -config ../../api/config.yml ../../api/openapi.yaml
type Server struct {
	AppName             string
	AuthService         service.TelegramAuthService
	ProductService      service.ProductService
	LLMService          service.LLMService
	FavouriteService    service.FavouriteService
	StoreRepository     repository.StoreRepository
	NotificationService service.NotificationService
}

func NewServer(config *config.Config, auth service.TelegramAuthService, prod service.ProductService,
	llm service.LLMService, fav service.FavouriteService, not service.NotificationService) (*Server, error) {
	repo, err := repository.NewStoreRepository(config)
	if err != nil {
		return nil, fmt.Errorf("failed to create store repository: %w", err)
	}

	return &Server{
		AppName:             config.Server.AppName,
		AuthService:         auth,
		ProductService:      prod,
		LLMService:          llm,
		FavouriteService:    fav,
		StoreRepository:     repo,
		NotificationService: not,
	}, nil
}

func (s *Server) PostAuthTelegram(ctx context.Context, req PostAuthTelegramRequestObject) (PostAuthTelegramResponseObject, error) {
	tracer := otel.Tracer(s.AppName)
	ctx, span := tracer.Start(ctx, "PostAuthTelegram")
	defer span.End()

	if req.Body == nil {
		return PostAuthTelegram400Response{}, fmt.Errorf("empty request body")
	}

	// Формируем данные из запроса.
	data := service.TelegramAuthData{
		ID:        int64(req.Body.Id),
		FirstName: req.Body.FirstName,
		LastName:  helper.StringFromPtr(req.Body.LastName),
		Username:  helper.StringFromPtr(req.Body.Username),
		PhotoURL:  helper.StringFromPtr(req.Body.PhotoUrl),
		AuthDate:  int64(req.Body.AuthDate),
		Hash:      req.Body.Hash,
	}

	// Проверка входящих данных от Telegram.
	if err := s.AuthService.ValidateAuth(data); err != nil {
		return PostAuthTelegram400Response{}, fmt.Errorf("auth validation error: %w", err)
	}

	// Проверяем, существует ли пользователь.
	exists, err := s.StoreRepository.UserExists(ctx, data.ID)
	if err != nil {
		return PostAuthTelegram400Response{}, fmt.Errorf("failed to check if user exists: %w", err)
	}

	if !exists {
		// Если пользователь не существует – создаём новую запись.
		newUser := domain.User{
			UserID:    data.ID,
			FirstName: data.FirstName,
			LastName:  data.LastName,
			Username:  data.Username,
			PhotoURL:  data.PhotoURL,
		}
		if err := s.StoreRepository.CreateUser(ctx, newUser); err != nil {
			return PostAuthTelegram400Response{}, fmt.Errorf("failed to create user: %w", err)
		}
	} else {
		// Если пользователь существует – обновляем его данные.
		updatedUser := domain.User{
			UserID:    data.ID,
			FirstName: data.FirstName,
			LastName:  data.LastName,
			Username:  data.Username,
			PhotoURL:  data.PhotoURL,
		}
		if err := s.StoreRepository.UpdateUser(ctx, updatedUser); err != nil {
			return PostAuthTelegram400Response{}, fmt.Errorf("failed to update user: %w", err)
		}
	}

	// Генерация JWT-токена.
	token, err := s.AuthService.GenerateToken(data)
	if err != nil {
		return PostAuthTelegram400Response{}, fmt.Errorf("failed to generate token: %w", err)
	}

	return PostAuthTelegram200JSONResponse{
		Token:   &token,
		Message: helper.PtrFromString("Authentication successful"),
	}, nil
}

func (s *Server) PostFavouritesAdd(ctx context.Context, req PostFavouritesAddRequestObject) (PostFavouritesAddResponseObject, error) {
	tracer := otel.Tracer(s.AppName)
	ctx, span := tracer.Start(ctx, "PostFavouritesAdd")
	defer span.End()

	if req.Body == nil {
		return PostFavouritesAdd400Response{}, fmt.Errorf("empty request body")
	}
	userID, ok := auth.GetUserIDFromContext(ctx)
	if !ok {
		return PostFavouritesAdd400Response{}, fmt.Errorf("user not authenticated")
	}
	err := s.FavouriteService.AddToFavourites(ctx, userID, req.Body.ProductId)
	if err != nil {
		return PostFavouritesAdd400Response{}, err
	}
	return PostFavouritesAdd200JSONResponse{Message: helper.PtrFromString("AbstractProduct added to favourites")}, nil
}

func (s *Server) DeleteFavouritesDelete(ctx context.Context, req DeleteFavouritesDeleteRequestObject) (DeleteFavouritesDeleteResponseObject, error) {
	tracer := otel.Tracer(s.AppName)
	ctx, span := tracer.Start(ctx, "DeleteFavouritesDelete")
	defer span.End()

	userID, ok := auth.GetUserIDFromContext(ctx)
	if !ok {
		return DeleteFavouritesDelete200JSONResponse{}, fmt.Errorf("user not authenticated")
	}
	err := s.FavouriteService.DeleteFromFavourites(ctx, userID, req.Params.ProductId)
	if err != nil {
		return DeleteFavouritesDelete200JSONResponse{}, err
	}
	return DeleteFavouritesDelete200JSONResponse{Message: helper.PtrFromString("AbstractProduct removed from favourites")}, nil
}

func (s *Server) GetFavouritesGet(ctx context.Context, req GetFavouritesGetRequestObject) (GetFavouritesGetResponseObject, error) {
	tracer := otel.Tracer(s.AppName)
	ctx, span := tracer.Start(ctx, "GetFavouritesGet")
	defer span.End()

	userID, ok := auth.GetUserIDFromContext(ctx)
	if !ok {
		return GetFavouritesGet400Response{}, fmt.Errorf("user not authenticated")
	}
	prod, err := s.FavouriteService.GetFavouriteProduct(ctx, userID, req.Params.ProductId)
	if err != nil {
		return GetFavouritesGet400Response{}, err
	}
	var content map[string]interface{}
	if err := json.Unmarshal([]byte(prod.PriorityData), &content); err != nil {
		content = make(map[string]interface{})
	}
	productResp := struct {
		Content *map[string]interface{} `json:"content,omitempty"`
		Id      *string                 `json:"id,omitempty"`
		Name    *string                 `json:"name,omitempty"`
		Photos  *[]string               `json:"photos,omitempty"`
		Prices  *[]struct {
			Market *string  `json:"market,omitempty"`
			Price  *float32 `json:"price,omitempty"`
			Url    *string  `json:"url,omitempty"`
		} `json:"prices,omitempty"`
	}{
		Content: &content,
		Id:      &prod.ID,
		Name:    &prod.Name,
	}
	return GetFavouritesGet200JSONResponse{
		Product: &productResp,
	}, nil
}

func (s *Server) GetFavouritesList(ctx context.Context, _ GetFavouritesListRequestObject) (GetFavouritesListResponseObject, error) {
	tracer := otel.Tracer(s.AppName)
	ctx, span := tracer.Start(ctx, "GetFavouritesList")
	defer span.End()

	userID, ok := auth.GetUserIDFromContext(ctx)
	if !ok {
		return GetFavouritesList400Response{}, fmt.Errorf("user not authenticated")
	}
	products, err := s.FavouriteService.ListFavourites(ctx, userID)
	if err != nil {
		return GetFavouritesList400Response{}, err
	}
	var list []struct {
		Content *map[string]interface{} `json:"content,omitempty"`
		Id      *string                 `json:"id,omitempty"`
		Name    *string                 `json:"name,omitempty"`
		Photos  *[]string               `json:"photos,omitempty"`
		Prices  *[]struct {
			Market *string  `json:"market,omitempty"`
			Price  *float32 `json:"price,omitempty"`
			Url    *string  `json:"url,omitempty"`
		} `json:"prices,omitempty"`
	}
	for _, p := range products {
		var cnt map[string]interface{}
		if err := json.Unmarshal([]byte(p.PriorityData), &cnt); err != nil {
			cnt = make(map[string]interface{})
		}
		list = append(list, struct {
			Content *map[string]interface{} `json:"content,omitempty"`
			Id      *string                 `json:"id,omitempty"`
			Name    *string                 `json:"name,omitempty"`
			Photos  *[]string               `json:"photos,omitempty"`
			Prices  *[]struct {
				Market *string  `json:"market,omitempty"`
				Price  *float32 `json:"price,omitempty"`
				Url    *string  `json:"url,omitempty"`
			} `json:"prices,omitempty"`
		}{
			Id:      &p.ID,
			Name:    &p.Name,
			Content: &cnt,
		})
	}
	return GetFavouritesList200JSONResponse{Favourites: &list}, nil
}

func (s *Server) GetGetAbstractProduct(ctx context.Context, request GetGetAbstractProductRequestObject) (GetGetAbstractProductResponseObject, error) {
	tracer := otel.Tracer(s.AppName)
	ctx, span := tracer.Start(ctx, "GetGetProduct")
	defer span.End()

	prod, err := s.ProductService.GetProductByID(ctx, request.Params.Id)
	if err != nil {
		return GetGetAbstractProduct400Response{}, err
	}
	var content map[string]interface{}
	if err := json.Unmarshal([]byte(prod.PriorityData), &content); err != nil {
		return GetGetAbstractProduct400Response{}, fmt.Errorf("failed to parse product content: %w", err)
	}
	productResp := struct {
		Content   *map[string]interface{} `json:"content,omitempty"`
		CreatedAt *time.Time              `json:"created_at,omitempty"`
		Id        *string                 `json:"id,omitempty"`
		Name      *string                 `json:"name,omitempty"`
		UpdatedAt *time.Time              `json:"updated_at,omitempty"`
	}{
		Id:        &prod.ID,
		Name:      &prod.Name,
		Content:   &content,
		CreatedAt: &prod.CreatedAt,
		UpdatedAt: &prod.UpdatedAt,
	}
	return GetGetAbstractProduct200JSONResponse{
		Product: &productResp,
	}, nil
}

func (s *Server) GetGetParsedProducts(ctx context.Context, req GetGetParsedProductsRequestObject) (GetGetParsedProductsResponseObject, error) {
	abstractID := req.Params.AbstractId
	if abstractID == "" {
		return GetGetParsedProducts400Response{}, fmt.Errorf("abstractId parameter is required")
	}

	parsedProducts, err := s.ProductService.GetParsedProductsByAbstractID(ctx, abstractID)
	if err != nil {
		return GetGetParsedProducts400Response{}, fmt.Errorf("failed to get parsed products: %w", err)
	}

	// Формируем список для ответа.
	var items []struct {
		Id         *string   `json:"id,omitempty"`
		Market     *string   `json:"market,omitempty"`
		PhotoUrl   *[]string `json:"photo_url,omitempty"`
		Price      *float64  `json:"price,omitempty"`
		ProductUrl *string   `json:"product_url,omitempty"`
	}
	for _, pp := range parsedProducts {
		items = append(items, struct {
			Id         *string   `json:"id,omitempty"`
			Market     *string   `json:"market,omitempty"`
			PhotoUrl   *[]string `json:"photo_url,omitempty"`
			Price      *float64  `json:"price,omitempty"`
			ProductUrl *string   `json:"product_url,omitempty"`
		}{
			Id:         helper.PtrFromString(pp.ID),
			Price:      &pp.Price,
			ProductUrl: helper.PtrFromString(pp.ProductURL),
			Market:     helper.PtrFromString(pp.MarketName),
			PhotoUrl:   &pp.PhotoUrls,
		})
	}

	return GetGetParsedProducts200JSONResponse{
		ParsedProducts: &items,
	}, nil
}

func (s *Server) PostLlmRequest(ctx context.Context, req PostLlmRequestRequestObject) (PostLlmRequestResponseObject, error) {
	tracer := otel.Tracer(s.AppName)
	ctx, span := tracer.Start(ctx, "PostLlmRequest")
	defer span.End()

	if req.Body == nil || req.Body.Query == "" {
		return PostLlmRequest400Response{}, fmt.Errorf("query is required")
	}
	llmResp, err := s.LLMService.ProcessQuery(ctx, req.Body.Query, 20)
	if err != nil {
		return PostLlmRequest400Response{}, err
	}
	products, err := s.ProductService.GetProductsByIDs(ctx, llmResp.Ids)
	if err != nil {
		return PostLlmRequest400Response{}, err
	}
	var details []struct {
		Content   *map[string]interface{} `json:"content,omitempty"`
		CreatedAt *time.Time              `json:"created_at,omitempty"`
		Id        *string                 `json:"id,omitempty"`
		MinPrice  *float64                `json:"min_price,omitempty"`
		Name      *string                 `json:"name,omitempty"`
		PhotoUrl  *string                 `json:"photo_url,omitempty"`
		UpdatedAt *time.Time              `json:"updated_at,omitempty"`
	}
	for _, p := range products {
		var cnt map[string]interface{}
		if err := json.Unmarshal([]byte(p.PriorityData), &cnt); err != nil {
			cnt = make(map[string]interface{})
		}
		parsedProducts, _ := s.ProductService.GetParsedProductsByAbstractID(ctx, p.ID)
		var minPrice = 0.0
		var firstPhoto = ""
		if (len(parsedProducts)) > 0 {
			minPrice = parsedProducts[0].Price
			if len(parsedProducts[0].PhotoUrls) > 0 {
				firstPhoto = parsedProducts[0].PhotoUrls[0]
			}
		}

		for _, pp := range parsedProducts {
			minPrice = min(minPrice, pp.Price)
		}
		details = append(details, struct {
			Content   *map[string]interface{} `json:"content,omitempty"`
			CreatedAt *time.Time              `json:"created_at,omitempty"`
			Id        *string                 `json:"id,omitempty"`
			MinPrice  *float64                `json:"min_price,omitempty"`
			Name      *string                 `json:"name,omitempty"`
			PhotoUrl  *string                 `json:"photo_url,omitempty"`
			UpdatedAt *time.Time              `json:"updated_at,omitempty"`
		}{
			Id:        &p.ID,
			Name:      &p.Name,
			Content:   &cnt,
			CreatedAt: &p.CreatedAt,
			UpdatedAt: &p.UpdatedAt,
			MinPrice:  &minPrice,
			PhotoUrl:  &firstPhoto,
		})
	}
	return PostLlmRequest200JSONResponse{
		Comment:  &llmResp.Comment,
		Products: &details,
	}, nil
}

func (s *Server) PostPriceChange(ctx context.Context, req PostPriceChangeRequestObject) (PostPriceChangeResponseObject, error) {
	tracer := otel.Tracer(s.AppName)
	ctx, span := tracer.Start(ctx, "PostPriceChange")
	defer span.End()

	if req.Body == nil {
		return PostPriceChange400Response{}, fmt.Errorf("empty request")
	}
	body := req.Body
	// Можно добавить валидацию входных данных, например, проверить, что oldPrice и newPrice > 0.

	// Вызов PublishPriceChange. Можно расширить метод, чтобы передавать marketName, если нужно.
	if err := s.NotificationService.PublishPriceChange(ctx, body.ProductId, body.OldPrice, body.NewPrice, helper.StringFromPtr(body.MarketName)); err != nil {
		return PostPriceChange500Response{}, fmt.Errorf("failed to publish price change: %w", err)
	}
	return PostPriceChange200JSONResponse{
		Message: helper.PtrFromString("Notification published successfully"),
	}, nil
}
