package repository

import (
	"context"
	"fmt"
	"model/config"
	"time"

	"gorm.io/driver/postgres"
	"gorm.io/gorm"

	"model/internal/domain"
)

type StoreRepository interface {
	// Продукты
	GetProductByID(ctx context.Context, id string) (*domain.AbstractProduct, error)
	GetParsedProductsByAbstractID(ctx context.Context, abstractID string) ([]domain.ParsedProduct, error)
	GetProductsByIDs(ctx context.Context, ids []string) ([]domain.AbstractProduct, error)
	GetProductsByQuery(ctx context.Context, query string, limit int) ([]domain.AbstractProduct, error)
	// Избранное
	AddFavourite(ctx context.Context, fav domain.Favourite) error
	DeleteFavourite(ctx context.Context, userID, productID string) error
	GetFavouriteProduct(ctx context.Context, userID, productID string) (*domain.AbstractProduct, error)
	ListFavourites(ctx context.Context, userID string) ([]domain.AbstractProduct, error)
	ListFavouriteUserIDsByProduct(ctx context.Context, productID string) ([]string, error)
	// Пользователи
	GetUserByID(ctx context.Context, id int64) (*domain.User, error)
	CreateUser(ctx context.Context, user domain.User) error
	UpdateUser(ctx context.Context, user domain.User) error
	UserExists(ctx context.Context, id int64) (bool, error)
}

type storeRepository struct {
	db       *gorm.DB
	embedder string
}

func NewStoreRepository(cfg *config.Config) (StoreRepository, error) {
	db, err := gorm.Open(postgres.Open(cfg.DB.ConnectionString()), &gorm.Config{})
	if err != nil {
		return nil, fmt.Errorf("failed to connect to DB: %w", err)
	}
	return &storeRepository{db: db, embedder: cfg.LLM.Embedder}, nil
}

// 1. Получение абстрактного товара по id.
func (r *storeRepository) GetProductByID(ctx context.Context, id string) (*domain.AbstractProduct, error) {
	var product domain.AbstractProduct
	sqlQuery := `
		SELECT abstract_product_id as id, name, priority_data as content, created_at, updated_at 
		FROM abstract_products 
		WHERE abstract_product_id = ?
	`
	if err := r.db.WithContext(ctx).Raw(sqlQuery, id).Scan(&product).Error; err != nil {
		return nil, err
	}
	return &product, nil
}

// 2. Получение списка parsed_products для абстрактного товара, включая данные о магазине.
// Предполагается, что таблица parsed_products теперь имеет столбец category_id.
func (r *storeRepository) GetParsedProductsByAbstractID(ctx context.Context, abstractID string) ([]domain.ParsedProduct, error) {
	var results []domain.ParsedProduct
	sqlQuery := `
		SELECT 
			pp.parsed_product_id as id, 
			pp.price, 
			pp.product_url,
			m.name as market_name,
			COALESCE(array_agg(ph.photo_url) FILTER (WHERE ph.photo_url IS NOT NULL), '{}') as photo_urls
		FROM parsed_products pp
		LEFT JOIN market_categories mc ON mc.category_id = pp.market_category_id
		LEFT JOIN markets m ON mc.market_id = m.market_id
		LEFT JOIN product_photos ph ON ph.parsed_product_id = pp.parsed_product_id
		WHERE pp.abstract_product_id = ?
		GROUP BY pp.parsed_product_id, pp.price, pp.product_url, m.name
	`
	if err := r.db.WithContext(ctx).Raw(sqlQuery, abstractID).Scan(&results).Error; err != nil {
		return nil, fmt.Errorf("failed to get parsed products: %w", err)
	}
	return results, nil
}

// 3. Получение списка абстрактных товаров по списку id.
func (r *storeRepository) GetProductsByIDs(ctx context.Context, ids []string) ([]domain.AbstractProduct, error) {
	var products []domain.AbstractProduct
	sqlQuery := `
		SELECT abstract_product_id as id, name, priority_data as content, created_at, updated_at
		FROM abstract_products
		WHERE abstract_product_id IN (?)
	`
	if err := r.db.WithContext(ctx).Raw(sqlQuery, ids).Scan(&products).Error; err != nil {
		return nil, fmt.Errorf("failed to get products by IDs: %w", err)
	}
	return products, nil
}

// 4. Поиск товаров по запросу с использованием эмбеддингов.
// Здесь мы объединяем таблицы: products_embeddings_store -> parsed_products -> abstract_products.
func (r *storeRepository) GetProductsByQuery(ctx context.Context, queryStr string, limit int) ([]domain.AbstractProduct, error) {
	var products []domain.AbstractProduct
	sqlQuery := `
		SELECT ap.abstract_product_id as id, ap.name, ap.priority_data as content
		FROM products_embeddings_store pes
		JOIN abstract_products ap ON pes.abstract_product_id = ap.abstract_product_id
		ORDER BY pes.embedding <=> ai.ollama_embed(?, ?, host => 'http://ollama:11434')
		LIMIT ?
	`
	if err := r.db.WithContext(ctx).Raw(sqlQuery, r.embedder, queryStr, limit).Scan(&products).Error; err != nil {
		return nil, fmt.Errorf("similarity search error: %w", err)
	}
	return products, nil
}

// Wishlist

func (r *storeRepository) AddFavourite(ctx context.Context, fav domain.Favourite) error {
	sqlQuery := `INSERT INTO wish_list_items (user_id, product_id) VALUES (?, ?)`
	if err := r.db.WithContext(ctx).Exec(sqlQuery, fav.UserID, fav.ProductID).Error; err != nil {
		return fmt.Errorf("failed to add favourite: %w", err)
	}
	return nil
}

func (r *storeRepository) DeleteFavourite(ctx context.Context, userID, productID string) error {
	sqlQuery := `DELETE FROM wish_list_items WHERE user_id = ? AND product_id = ?`
	if err := r.db.WithContext(ctx).Exec(sqlQuery, userID, productID).Error; err != nil {
		return fmt.Errorf("failed to delete favourite: %w", err)
	}
	return nil
}

func (r *storeRepository) GetFavouriteProduct(ctx context.Context, userID, productID string) (*domain.AbstractProduct, error) {
	var product domain.AbstractProduct
	sqlQuery := `
		SELECT p.abstract_product_id as id, p.name, p.priority_data as content
		FROM wish_list_items w
		JOIN abstract_products p ON p.abstract_product_id = w.product_id
		WHERE w.user_id = ? AND w.product_id = ?
	`
	if err := r.db.WithContext(ctx).Raw(sqlQuery, userID, productID).Scan(&product).Error; err != nil {
		return nil, err
	}
	return &product, nil
}

func (r *storeRepository) ListFavourites(ctx context.Context, userID string) ([]domain.AbstractProduct, error) {
	var products []domain.AbstractProduct
	sqlQuery := `
		SELECT p.abstract_product_id as id, p.name, p.priority_data as content
		FROM wish_list_items w
		JOIN abstract_products p ON p.abstract_product_id = w.product_id
		WHERE w.user_id = ?
	`
	if err := r.db.WithContext(ctx).Raw(sqlQuery, userID).Scan(&products).Error; err != nil {
		return nil, err
	}
	return products, nil
}

func (r *storeRepository) ListFavouriteUserIDsByProduct(ctx context.Context, productID string) ([]string, error) {
	var userIDs []string
	query := "SELECT user_id FROM wish_list_items WHERE product_id = ?"
	if err := r.db.WithContext(ctx).Raw(query, productID).Pluck("user_id", &userIDs).Error; err != nil {
		return nil, err
	}
	return userIDs, nil
}

// Пользователи

func (r *storeRepository) GetUserByID(ctx context.Context, id int64) (*domain.User, error) {
	var user domain.User
	if err := r.db.WithContext(ctx).First(&user, "user_id = ?", id).Error; err != nil {
		return nil, err
	}
	return &user, nil
}

func (r *storeRepository) CreateUser(ctx context.Context, user domain.User) error {
	// Устанавливаем время создания и обновления, если не заданы.
	now := time.Now()
	if user.CreatedAt.IsZero() {
		user.CreatedAt = now
	}
	if user.UpdatedAt.IsZero() {
		user.UpdatedAt = now
	}
	return r.db.WithContext(ctx).Create(&user).Error
}

func (r *storeRepository) UpdateUser(ctx context.Context, user domain.User) error {
	// Обновляем время изменения.
	user.UpdatedAt = time.Now()
	return r.db.WithContext(ctx).Save(&user).Error
}

func (r *storeRepository) UserExists(ctx context.Context, id int64) (bool, error) {
	var count int64
	// Используем модель domain.User для подсчёта записей с заданным user_id.
	if err := r.db.WithContext(ctx).Model(&domain.User{}).Where("user_id = ?", id).Count(&count).Error; err != nil {
		return false, err
	}
	return count > 0, nil
}
