package service

import (
	"context"
	"encoding/json"
	"fmt"
	"model/config"
	"strings"

	"model/internal/ml"
)

// LLMResponse представляет структуру ответа от LLM.
type LLMResponse struct {
	Ids     []string `json:"Ids"`
	Comment string   `json:"comment"`
}

type LLMService interface {
	ProcessQuery(ctx context.Context, query string, limit int) (*LLMResponse, error)
}

type llmService struct {
	client         ml.LangchangoClient
	productService ProductService
	cfg            *config.Config
}

func NewLLMService(ctx context.Context, cfg *config.Config, ps ProductService) (LLMService, error) {
	client, err := ml.NewLangchangoClient(cfg.LLM.ModelName, cfg.LLM.APIKey)
	if err != nil {
		return nil, err
	}
	return &llmService{
		client:         client,
		productService: ps,
		cfg:            cfg,
	}, nil
}

func (s *llmService) ProcessQuery(ctx context.Context, query string, limit int) (*LLMResponse, error) {
	// Проверка запроса
	if query == "" {
		return nil, fmt.Errorf("query cannot be empty")
	}

	// Получаем список абстрактных товаров, найденных по запросу
	products, err := s.productService.GetProductsByQuery(ctx, query, limit)
	if err != nil {
		return nil, fmt.Errorf("failed to get products: %w", err)
	}

	// Если товаров не найдено, возвращаем стандартный ответ
	if len(products) == 0 {
		return &LLMResponse{
			Ids:     []string{},
			Comment: "По вашему запросу товары не найдены",
		}, nil
	}

	// Строим контекст доступных товаров
	var sb strings.Builder
	sb.WriteString("Доступные товары:\n")
	for i, p := range products {
		sb.WriteString(fmt.Sprintf("%d. ID: %s\n   Название: %s\n   Характеристики: %s\n", i+1, p.ID, p.Name, p.PriorityData))
	}

	// Формируем промпт с четкими инструкциями
	prompt := fmt.Sprintf(`Ты ассистент по поиску товаров. 
Анализируй запрос пользователя и список доступных товаров, чтобы выбрать наиболее релевантные. Не придумывай товары только выбирай из доступных

- Список товаров:
%s

На вход тебе от пользователя придёт запрос с тем, что он хочет найти.

Требования к ответу:
1. Ответ должен быть валидным JSON объектом. Верни ответ в виде чистого JSON без дополнительного форматирования, без обратных кавычек и слова json, сразу {}.
2. JSON объект должен содержать два поля: "Ids" – массив строк (UUID товаров), и "comment" – строку с обоснованием выбора.
3. Используй только ID товаров из предоставленного списка.
4. Если подходящих товаров нет, верни {"Ids": [], "comment": "Товары не найдены"}.
5. Верни только JSON без лишнего текста.
6. Формат ответа: {"Ids": ["id1", "id2"], "comment": "Обоснование выбора"}`,
		sb.String())

	response, err := s.client.GenerateContent(ctx, prompt, query)
	if err != nil {
		return nil, fmt.Errorf("failed to generate content: %w", err)
	}

	var llmResp LLMResponse
	if err := json.Unmarshal([]byte(response), &llmResp); err != nil {
		return nil, fmt.Errorf("failed to parse LLM response: %w", err)
	}

	// Валидация ответа
	if llmResp.Ids == nil {
		llmResp.Ids = []string{}
	}

	return &llmResp, nil
}
