package service

import (
	"context"
	"encoding/json"
	"fmt"
	"log"
	"strconv"
	"time"

	"model/internal/domain"
	"model/internal/kafka"
	"model/internal/repository"
)

// NotificationService описывает методы сервиса уведомлений.
type NotificationService interface {
	// PublishPriceChange публикует сообщение об изменении цены в Kafka.
	PublishPriceChange(ctx context.Context, productID string, oldPrice, newPrice int, marketName string) error
	// StartConsumer запускает процесс обработки сообщений из Kafka.
	StartConsumer(ctx context.Context)
}
type notificationService struct {
	producer kafka.Producer
	consumer kafka.Consumer
	repo     repository.StoreRepository
	notifier TelegramNotifier
	topic    string
}

// NewNotificationService создаёт новый экземпляр NotificationService.
func NewNotificationService(producer kafka.Producer, consumer kafka.Consumer, repo repository.StoreRepository, notifier TelegramNotifier, topic string) NotificationService {
	return &notificationService{
		producer: producer,
		consumer: consumer,
		repo:     repo,
		notifier: notifier,
		topic:    topic,
	}
}

// PublishPriceChange публикует событие изменения цены в Kafka.
func (s *notificationService) PublishPriceChange(ctx context.Context, productID string, oldPrice, newPrice int, marketName string) error {
	notification := domain.PriceChangeNotification{
		ProductID: productID,
		OldPrice:  oldPrice,
		NewPrice:  newPrice,
		Timestamp: time.Now(),
		Market:    marketName,
	}
	payload, err := json.Marshal(notification)
	if err != nil {
		return fmt.Errorf("failed to marshal notification: %w", err)
	}

	msg := kafka.Message{
		Value: payload,
		Time:  time.Now(),
	}

	if err := s.producer.Write(ctx, msg); err != nil {
		return fmt.Errorf("failed to publish notification: %w", err)
	}
	log.Printf("Published price change notification for product %s", productID)
	return nil
}

// StartConsumer запускает обработку сообщений из Kafka.
func (s *notificationService) StartConsumer(ctx context.Context) {
	go s.consumer.Start(ctx, func(ctx context.Context, msg kafka.Message) error {
		var notification domain.PriceChangeNotification
		if err := json.Unmarshal(msg.Value, &notification); err != nil {
			log.Printf("Failed to unmarshal notification: %v", err)
			return err
		}
		log.Printf("Consumed price change notification: %+v", notification)

		// Получаем список userID, отслеживающих данный продукт.
		userIDs, err := s.repo.ListFavouriteUserIDsByProduct(ctx, notification.ProductID)
		if err != nil {
			log.Printf("Failed to get user IDs for product %s: %v", notification.ProductID, err)
			return err
		}

		product, err := s.repo.GetProductByID(ctx, notification.ProductID)
		if err != nil {
			log.Printf("Failed to get product info for productID %s: %v", notification.ProductID, err)
			return err
		}
		parsedProducts, err := s.repo.GetParsedProductsByAbstractID(ctx, notification.ProductID)
		if err != nil {
			log.Printf("Failed to get parsed products for productID %s: %v", notification.ProductID, err)
			return err
		}
		url := ""
		for _, p := range parsedProducts {
			if p.MarketName == notification.Market {
				url = p.ProductURL
				break
			}
		}

		for _, userID := range userIDs {
			// Преобразуем строковый userID в int64 для получения данных из БД.
			uid, err := parseUserID(userID)
			if err != nil {
				log.Printf("Invalid userID format (%s): %v", userID, err)
				continue
			}
			user, err := s.repo.GetUserByID(ctx, uid)
			if err != nil {
				log.Printf("Failed to get user info for userID %s: %v", userID, err)
				continue
			}
			// Формируем персонализированное сообщение.
			message := fmt.Sprintf("Привет, %s! Цена на продукт %s в магазине %s изменилась: с %d до %d\n%s",
				user.FirstName, product.Name, notification.Market, notification.OldPrice, notification.NewPrice,
				url)
			if err := s.notifier.SendNotification(userID, message); err != nil {
				log.Printf("Failed to send notification to user %s: %v", userID, err)
			}
		}

		if err := s.consumer.Commit(ctx, msg); err != nil {
			log.Printf("Failed to commit message: %v", err)
			return err
		}
		return nil
	})
}

// parseUserID преобразует строку в int64.
func parseUserID(userID string) (int64, error) {
	return strconv.ParseInt(userID, 10, 64)
}
