package service

import (
	"crypto/hmac"
	"crypto/sha256"
	"encoding/hex"
	"fmt"
	"model/config"
	"net/url"
	"sort"
	"strconv"
	"time"

	"github.com/golang-jwt/jwt/v4"
)

// TelegramAuthData – данные, получаемые от Telegram.
type TelegramAuthData struct {
	ID        int64  `json:"id"`
	FirstName string `json:"first_name"`
	LastName  string `json:"last_name"`
	Username  string `json:"username"`
	PhotoURL  string `json:"photo_url"`
	AuthDate  int64  `json:"auth_date"`
	Hash      string `json:"hash"`
}

// TelegramAuthService описывает методы авторизации через Telegram.
type TelegramAuthService interface {
	ValidateAuth(data TelegramAuthData) error
	GenerateToken(data TelegramAuthData) (string, error)
}

type telegramAuthService struct {
	botToken  string
	jwtSecret string
}

// NewTelegramAuthService создаёт новый сервис авторизации, используя настройки из конфигурации.
func NewTelegramAuthService(cfg *config.Config) TelegramAuthService {
	return &telegramAuthService{
		botToken:  cfg.Telegram.BotToken,
		jwtSecret: cfg.Telegram.JWTSecret,
	}
}

// ValidateAuth проверяет корректность данных, полученных от Telegram.
func (s *telegramAuthService) ValidateAuth(data TelegramAuthData) error {
	values := url.Values{}
	values.Set("id", strconv.FormatInt(data.ID, 10))
	values.Set("first_name", data.FirstName)
	if data.LastName != "" {
		values.Set("last_name", data.LastName)
	}
	if data.Username != "" {
		values.Set("username", data.Username)
	}
	if data.PhotoURL != "" {
		values.Set("photo_url", data.PhotoURL)
	}
	values.Set("auth_date", strconv.FormatInt(data.AuthDate, 10))

	// Формируем строку для проверки, сортируя ключи.
	var keys []string
	for k := range values {
		keys = append(keys, k)
	}
	sort.Strings(keys)

	var dataCheckString string
	for i, k := range keys {
		if i > 0 {
			dataCheckString += "\n"
		}
		dataCheckString += fmt.Sprintf("%s=%s", k, values.Get(k))
	}

	// Вычисляем ожидаемый хэш.
	secretKey := sha256.Sum256([]byte(s.botToken))
	mac := hmac.New(sha256.New, secretKey[:])
	mac.Write([]byte(dataCheckString))
	expectedHash := hex.EncodeToString(mac.Sum(nil))

	if expectedHash != data.Hash {
		return fmt.Errorf("invalid Telegram auth hash")
	}

	// Проверяем, что данные не устарели (более 24 часов).
	if time.Now().Unix()-data.AuthDate > 86400 {
		return fmt.Errorf("auth data is too old")
	}

	return nil
}

// GenerateToken генерирует JWT-токен для авторизованного пользователя.
func (s *telegramAuthService) GenerateToken(data TelegramAuthData) (string, error) {
	// Ожидается, что перед вызовом этой функции данные прошли валидацию.
	claims := jwt.MapClaims{
		"user_id":  data.ID,
		"username": data.Username,
		"exp":      time.Now().Add(24 * time.Hour).Unix(),
		"iat":      time.Now().Unix(),
	}
	token := jwt.NewWithClaims(jwt.SigningMethodHS256, claims)
	return token.SignedString([]byte(s.jwtSecret))
}
