-- Включаем расширения для генерации UUID и работы с векторами (если они ещё не включены)
CREATE EXTENSION IF NOT EXISTS "uuid-ossp";
CREATE EXTENSION IF NOT EXISTS vector;
CREATE EXTENSION IF NOT EXISTS pgcrypto;
CREATE EXTENSION IF NOT EXISTS vectorscale CASCADE;
CREATE EXTENSION IF NOT EXISTS ai CASCADE;


CREATE TABLE IF NOT EXISTS users (
                                     user_id BIGINT PRIMARY KEY,       -- Уникальный идентификатор пользователя (Telegram ID)
                                     first_name VARCHAR(255) NOT NULL,  -- Имя пользователя
                                     last_name VARCHAR(255),            -- Фамилия пользователя (опционально)
                                     username VARCHAR(255),             -- Никнейм в Telegram (опционально)
                                     photo_url TEXT,                    -- Ссылка на аватар (опционально)
                                     created_at TIMESTAMPTZ NOT NULL DEFAULT NOW(),  -- Дата регистрации
                                     updated_at TIMESTAMPTZ NOT NULL DEFAULT NOW()   -- Дата последнего обновления
);
-- 1. Таблица с «абстрактными» товарами (основная сущность для хранения сведений о товаре).
CREATE TABLE IF NOT EXISTS abstract_products (
                                                 abstract_product_id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
                                                 name               TEXT NOT NULL,
                                                 name_vector        VECTOR(1024),
                                                 priority           SMALLINT DEFAULT 0,
                                                 priority_data      text,
                                                 created_at         TIMESTAMPTZ DEFAULT NOW(),
                                                 updated_at         TIMESTAMPTZ DEFAULT NOW()
);

-- 2. Таблица категорий (иерархическая структура).
CREATE TABLE IF NOT EXISTS categories (
                                          category_id        UUID PRIMARY KEY DEFAULT gen_random_uuid(),
                                          parent_category_id UUID,
                                          name               TEXT NOT NULL,
                                          FOREIGN KEY (parent_category_id) REFERENCES categories(category_id)
);

-- 3. Таблица маркетов (продавцов, площадок).
CREATE TABLE IF NOT EXISTS markets (
                                       market_id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
                                       name      TEXT NOT NULL,
                                       base_url  TEXT,
                                       priority  SMALLINT DEFAULT 0
);

-- 4. Связующая таблица market_categories (Many-to-Many: market ↔ categories).
CREATE TABLE IF NOT EXISTS market_categories (
                                                 market_category_id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
                                                 category_id UUID NOT NULL,
                                                 name        TEXT NOT NULL,
                                                 lastUpdated TIMESTAMPTZ DEFAULT NOW(),
                                                 market_id   UUID NOT NULL,
                                                 url         TEXT,
                                                 FOREIGN KEY (market_id) REFERENCES markets(market_id),
                                                 FOREIGN KEY (category_id) REFERENCES categories(category_id)
);

-- 5. Таблица «спарсенных» товаров, ссылается на abstract_products.
CREATE TABLE IF NOT EXISTS parsed_products(
                                               parsed_product_id    UUID PRIMARY KEY DEFAULT gen_random_uuid(),
                                               abstract_product_id  UUID,
                                               market_category_id   UUID NOT NULL,
                                               price                NUMERIC,
                                               product_url          TEXT,
                                               price_history        jsonb,
                                               FOREIGN KEY (abstract_product_id) REFERENCES abstract_products(abstract_product_id),
                                               FOREIGN KEY (market_category_id) REFERENCES market_categories(market_category_id)
);

-- 6. Фотографии товаров, привязанные к abstract_products.
CREATE TABLE IF NOT EXISTS product_photos (
                                              photo_id   UUID PRIMARY KEY DEFAULT gen_random_uuid(),
                                              parsed_product_id UUID NOT NULL,
                                              photo_url  TEXT NOT NULL,
                                              FOREIGN KEY (parsed_product_id) REFERENCES parsed_products(parsed_product_id)
);

SELECT ai.create_vectorizer(
               'abstract_products'::regclass,
               destination => 'products_embeddings',
               embedding => ai.embedding_ollama('bge-m3', 1024),
               chunking => ai.chunking_recursive_character_text_splitter('priority_data')
       );


-- 8. Таблица списка желаемого (wish_list_items).
-- Предполагается, что таблица users уже существует с полем user_id (BIGINT).
CREATE TABLE IF NOT EXISTS wish_list_items (
                                               user_id    BIGINT NOT NULL,
                                               product_id UUID   NOT NULL,
                                               created_at TIMESTAMPTZ DEFAULT NOW(),
                                               PRIMARY KEY (user_id, product_id),
                                               FOREIGN KEY (user_id) REFERENCES users(user_id),
                                               FOREIGN KEY (product_id) REFERENCES abstract_products(abstract_product_id)
);
